require 'kubeclient'
require 'ocpclient/version'
require 'json'
require 'rest-client'

module Ocpclient
  # OCP Client
  class Client < Kubeclient::Client
    include Kubeclient::ClientMixin
    # Dynamically creating classes definitions (class Project, class BuildConfig, etc.),
    # The classes are extending RecursiveOpenStruct.
    # This cancels the need to define the classes
    # manually on every new entity addition,
    # and especially since currently the class body is empty
    ENTITY_TYPES = %w(Project ProjectRequest BuildConfig RoleBinding).map do |et|
      clazz = Class.new(RecursiveOpenStruct) do
        def initialize(hash = nil, args = {})
          args.merge!(recurse_over_arrays: true)
          super(hash, args)
        end
      end
      [Kubeclient.const_set(et, clazz), et]
    end

    Kubeclient::ClientMixin.define_entity_methods(ENTITY_TYPES)

    def initialize(
      uri,
      version = 'v1',
      **options
    )
      initialize_client(
        uri,
        '/oapi',
        version,
        options
      )
    end

    def all_entities
      retrieve_all_entities(ENTITY_TYPES)
    end
  end
end
